<?php

namespace Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\License;

class Config
{
    protected static $whitelistKeyNames = array('license_token', 'package_installation_date');

    protected $attributes;

    public function __construct(){
        $this->init();
    }

    public function init()
    {
        if (!isset($this->attributes)) {
            $this->attributes = array();

            $administration = \Administration::getSettings('PX_NeedToDoEvents');
            foreach (self::$whitelistKeyNames as $whitelistKeyName) {
                $key = 'PX_NeedToDoEvents' . '_' . $whitelistKeyName;
                if (isset($administration->settings[$key])) {
                    $this->attributes[$whitelistKeyName] = $administration->settings[$key];
                }
            }
        }
    }

    public function __get($name){
        if($name == 'package_installation_date' && !$this->isInstallationDateValid($this->attributes[$name])){
            $this->fixPluginInstallationDate();
        }
        if ($this->isNameInWhitelist($name) && array_key_exists($name, $this->attributes)) {
            return $this->attributes[$name];
        }
        return null;
    }

    public function __set($name, $value){
        if($this->isNameInWhitelist($name))
            $this->attributes[$name] = $value;
    }

    protected function isNameInWhitelist($name){
        return in_array($name, self::$whitelistKeyNames);
    }

    //TODO: Remove all this logic
    public function packageInstallationDateExists(){
        return !empty($this->attributes['package_installation_date']);
    }

    protected function savePackageInstallationDate($settingValue){
        $admin = \BeanFactory::newBean('Administration');
        $module = 'PX_NeedToDoEvents';
        $settingName = 'package_installation_date';
        $platform = 'base';
        $admin->saveSetting($module, $settingName, $settingValue, $platform);
        $this->attributes['package_installation_date'] = $settingValue;
    }

    public function initPackageInstallationDate(){
        $settingValue = $this->getCurrentDate();
        $this->savePackageInstallationDate($settingValue);
    }

    protected function getCurrentDate(){
        $timeDate = \TimeDate::getInstance();
        $now = $timeDate->getNow();
        $now->setTime(0, 0, 0);
        return $now->asDbDate();
    }

    protected function isInstallationDateValid($pluginInstallationDate){
        if(empty($pluginInstallationDate))
            return false;

        $timeDate = \TimeDate::getInstance();
        $pluginInstallationDate = $timeDate->fromDbDate($pluginInstallationDate);
        if(empty($pluginInstallationDate))
            return false;
        return true;
    }

    public function fixPluginInstallationDate(){
        $upgradeHistory = \BeanFactory::getBean('UpgradeHistory');
        if(!empty($upgradeHistory)){
            try {
                $query = new \SugarQuery();
                $query->from($upgradeHistory);
                $query->select(array('date_entered'));
                $query->where()->equals('id_name', 'integros_need_to_do_viewer');
                $pluginInstallationDate = $query->getOne();
                $timeDate = \TimeDate::getInstance();
                $pluginInstallationDate = $timeDate->fromDb($pluginInstallationDate);
                if(empty($pluginInstallationDate))
                    throw new \Exception('Dummy Exception');
                $pluginInstallationDate->setTime(0, 0, 0, 0);
                $this->savePackageInstallationDate($pluginInstallationDate->asDbDate());
            }catch(\Exception $e){
                $this->initPackageInstallationDate();
            }
        }else{
            $this->initPackageInstallationDate();
        }
    }

    public function licenseTokenExists(){
        return !empty($this->attributes['license_token']);
    }

    public function saveDemoLicenseToken(){
        $settingValue = $this->getInitialDemoToken();
        if(empty($settingValue))
            return;

        $admin = \BeanFactory::newBean('Administration');
        $module = 'PX_NeedToDoEvents';
        $settingName = 'license_token';
        $platform = 'base';
        $admin->saveSetting($module, $settingName, $settingValue, $platform);
        $this->attributes['license_token'] = $settingValue;
    }

    public function getInitialDemoToken(){
        $demoTokenFilePath = 'custom/PxRoutines/NeedToDoViewer/Package/License/Tokens/initial_demo_token.php';
        if(!\SugarAutoLoader::fileExists($demoTokenFilePath)){
            return '';
        }

        require $demoTokenFilePath;
        return empty($licenseToken) ? '' : $licenseToken;
    }
}
