<?php

namespace Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\License;

use Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\License\Validations\Validation;
use Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\License\Validations\DemoValidation;
use Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\License\Validations\StandardValidation;
use Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\License\Validations\Decorators\ProductValidationDecorator;

class ValidationFactory{
    /**
     * License Types
     */
    const DEMO_LICENSE_TOKEN = 'Demo';
    const STANDARD_LICENSE_TOKEN = 'Standard';

    /**
     * License Type key in Payload
     */
    const PK_LICENSE_TYPE = 'license_type';

    protected $tokenPayload;

    protected $packageInstallationDate;

    /**
     * StandardValidation constructor.
     * @param $tokenPayload - Decoded License Token
     * @param $packageInstallationDate - Date of Need To Do PKG installation.
     */
    public function __construct($tokenPayload, $packageInstallationDate){
        $this->tokenPayload = $tokenPayload;
        $this->packageInstallationDate = $packageInstallationDate;
    }

    /**
     * @return Validation
     */
    public function getValidation(){
        $licenseType = $this->getLicenseType();
        if($licenseType == self::DEMO_LICENSE_TOKEN){
            return new ProductValidationDecorator(
                new DemoValidation(
                    $this->tokenPayload,
                    $this->packageInstallationDate
                )
            );
        }
        return new ProductValidationDecorator(
            new StandardValidation($this->tokenPayload)
        );
    }

    protected function getLicenseType(){
        return !empty($this->tokenPayload[self::PK_LICENSE_TYPE]) ? $this->tokenPayload[self::PK_LICENSE_TYPE] : '';
    }
}
