({
    events: {
        'click .ntdv-collab-toggle-dialog': 'loadDialogHistory',
        'click .ntdv-collab-collapse-dialog': 'toggleDialog',

        'change div[data-placeholder]': 'checkPlaceholder',
        'keydown div[data-placeholder]': 'checkPlaceholder',
        'keypress div[data-placeholder]': 'checkPlaceholder',
        'input div[data-placeholder]': 'checkPlaceholder',
        'click .reply-btn': 'replyButtonClick',

        'click .ntdv-collab-dialog-favorite': 'toggleDialogAsFavorite',
        'click .btn[data-action="showMessageStatistics"]': 'showMessageStatistics',
    },
    /**
     * The plugins used by this view.
     */
    plugins: ['Dashlet', 'Pagination', 'Taggable'],

    className: 'block filtered ntdv-collaboration-dashlet',

    /**
     * The default settings for activity stream.
     *
     * @property {Object}
     */
    _defaultSettings: {
        limit: 5,
        auto_refresh: 0,
        filter_id: 'created_by_me',
    },

    dialogHistory: {},
    topicModel: {},
    messageIsProcessing: {},

    licenseStatus: null,
    licenseErrorLabel: '',

    initialize: function(opts) {
        this._super('initialize', [opts]);

        this.$el.scroll(_.bind(function(){
            this.onScroll(this);
        }, this));

        this.setCollectionOptions();
        this.initHandleBarHelpers();
    },

    /*******************************************************************************************************/
    /**************************************** INITIALIZE DASHLET *******************************************/
    /*******************************************************************************************************/
    /**
     * Init dashlet settings
     */
    initDashlet: function()
    {
        var options = {};
        var self = this;
        var refreshRate;

        options.limit = this.settings.get('limit') || this._defaultSettings.limit;
        this.settings.set('limit', options.limit);

        options.auto_refresh = this.settings.get('auto_refresh') || this._defaultSettings.auto_refresh;
        this.settings.set('auto_refresh', options.auto_refresh);

        options = _.extend({}, this.context.get('collectionOptions'), options);
        this.context.set('collectionOptions', options);

        if (!_.isEmpty(this.meta.currentFilterId) && _.isEmpty(this.meta.currentFilterId)) {
            this.settings.set('filter_id', this.meta.currentFilterId);
        }

        if (this.meta.config) {
            ////this._configureDashlet();
            this.listenTo(this.layout, 'init', this._addFilterComponent);
            this.listenTo(this.layout.context, 'filter:add', this.updateDashletFilterAndSave);
            this.listenTo(this.layout, 'dashletconfig:save', this._initializeSettings);
            this.layout.before('dashletconfig:save', function () {
                this.saveDashletFilter();
                this._initializeSettings();
                return false;
            }, this);
        } else{
            var filterId = this.settings.get('filter_id');
            if (!filterId || this.meta.preview) {
                this._displayDashlet();
                return;
            }

            var filters = app.data.createBeanCollection('Filters');
            filters.setModuleName('PX_NeedToDoViewerComments');
            filters.load({
                success: _.bind(function () {
                    if (this.disposed) {
                        return;
                    }
                    var filter = filters.collection.get(filterId);
                    var filterDef = filter && filter.get('filter_definition');
                    this._displayDashlet(filterDef);

                }, this),
                error: _.bind(function (err) {
                    if (this.disposed) {
                        return;
                    }
                    this._displayDashlet();
                }, this)
            });
        }

        this.layout.on('dashletconfig:save', function () {
            this._initializeSettings();
        }, this);
    },

    /**
     * Add Filters to Dashlet configuration
     * @private
     */
    _addFilterComponent: function() {
        var filterComponent = this.layout.getComponent('dashablelist-filter');
        if (filterComponent) {
            return;
        }

        if(!_.isUndefined(this.layout['initComponents'])) {
            this.layout.px_log = true;
            this.layout.initComponents([{
                layout: 'dashablelist-filter',
                loadModule: 'PX_NeedToDoViewerComments'
            }]);
        }else{
            this.layout._addComponentsFromDef([{
                layout: 'dashablelist-filter',
                loadModule: 'PX_NeedToDoViewerComments'
            }]);
        }
    },

    /**
     * Save new Filter for current dashlet
     * @param filterModel
     */
    updateDashletFilterAndSave: function(filterModel) {
        if(filterModel.id || filterModel instanceof Backbone.Model) {
            var id = filterModel.id || filterModel.get('id');
            this.settings.set('filter_id', id);
            this.dashModel.set('filter_id', id);
        }

        var componentType = this.dashModel.get('componentType') || 'view';

        if (!this.dashModel.get('componentType')) {
            this.dashModel.set('componentType', componentType);
        }
        app.drawer.close(this.dashModel);
        app.events.trigger('dashlet:filter:save', this.dashModel.get('module'));
    },

    saveDashletFilter: function() {
        var context = this.layout.context;

        if (context.editingFilter) {
            if (!context.editingFilter.get('name')) {
                context.editingFilter.set('name', app.lang.get('LBL_DASHLET') +
                    ': ' + this.settings.get('label'));
            }
            context.trigger('filter:create:save');
        } else {
            var filterId = context.get('currentFilterId'),
                obj = {id: filterId};
            this.updateDashletFilterAndSave(obj);
        }
    },

    /**
     * Initialize dashlet settings
     * @param model
     * @private
     */
    _initializeSettings: function(model) {
        if (!this.dashModel.get('label')) {
            this.dashModel.set('label', this.settings.get('label'));
        }
    },

    /**************************************** COLLECTION SETTINGS *******************************************/
    setCollectionOptions: function() {
        this.context.set('collectionOptions', this.getCollectionOptions());
    },

    getCollectionOptions: function(){
        var self = this,
            endpoint = function(method, model, options, callbacks) {
                var url = self.getCollectionFetchLink(options);
                return app.api.call("read", url, null, callbacks);
            },
            limit = !_.isEmpty(this.settings) ? this.settings.get('limit') : this._defaultSettings.limit;
        return {
            endpoint: endpoint,
            success: function(collection) {
                if(!_.isEmpty(self.$el)) {
                    self.$el.find('.block-footer').toggleClass('hide', (collection.length > 0));
                }
            },
            limit: limit
        };
    },

    getCollectionFetchLink: function(options){
        var url = app.api.buildURL('px-need-to-do-collaboration/dialog-list', '', {}, options['params']),
            limit = !_.isEmpty(this.settings) ? this.settings.get('limit') : this._defaultSettings.limit;;

        return url;
    },

    /*******************************************************************************************************/
    /**************************************** DISPLAY DASHLET *******************************************/
    /*******************************************************************************************************/
    _displayDashlet: function(filterDef) {
        if (filterDef) {
            this._applyFilterDef(filterDef);
            this.context.reloadData({'recursive': false});
        }
        this._startAutoRefresh();
    },

    _applyFilterDef: function(filterDef) {
        if (filterDef) {
            filterDef = _.isArray(filterDef) ? filterDef : [filterDef];
            var specialField = /^\$/,
                meta = app.metadata.getModule('PX_NeedToDoViewerComments');
            filterDef = _.filter(filterDef, function(def) {
                var fieldName = _.keys(def).pop();
                return specialField.test(fieldName) || meta.fields[fieldName];
            }, this);

            this.context.get('collection').filterDef = filterDef;
        }
    },

    loadData: function(options) {
        if (_.isUndefined(this.context.parent.get('layout'))) {
            return;
        }

        options = _.extend({}, options, this.context.get('collectionOptions'));

        if (this.collection) {
            var filters = app.data.createBeanCollection('Filters');
            var self = this;
            filters.setModuleName('PX_NeedToDoViewerComments');
            filters.load({
                success: _.bind(function() {
                    if (this.disposed)
                        return;

                    var filter = filters.collection.get(this.settings.get('filter_id'));
                    if (filter)
                        options.filter = filter.get('filter_definition');

                    self.collection.fetch(options);

                }, this),
                error: _.bind(function(err) {
                    if (this.disposed) {
                        return;
                    }
                    this._displayDashlet();
                }, this)
            });
        }
    },

    /**
     * Bind data changes and replies adding
     */
    bindDataChange: function() {
        if (this.collection) {
            this.collection.on('add', function(model) {
                this.renderDialog(model);

                this.$el.find('.block-footer').addClass('hide');
            }, this);
            this.collection.on('reset', function() {
                this.disposeAllHistory();

                this.collection.each(function(model) {
                    this.renderDialog(model);
                }, this);

                this.$el.find('.block-footer').toggleClass('hide', !this.collection.length);
            }, this);
        }
    },

    _startAutoRefresh: function() {
        var refreshRate = parseInt(this.settings.get('auto_refresh'), 10);
        if (refreshRate) {
            this._stopAutoRefresh();
            this._timerId = setInterval(_.bind(function() {
                //this.context.resetLoadFlag();
                this.layout.loadData();
            }, this), refreshRate * 1000 * 60);
        }
    },

    _stopAutoRefresh: function() {
        if (this._timerId) {
            clearInterval(this._timerId);
        }
    },

    paginate: function(options) {
        options = _.extend(options, this.getCollectionOptions(), {add: true});
        this.collection.paginate(options);
    },

    /**
     * Skip rendering on pagination update
     * This is a hack to prevent dashlet re-render
     * @param options
     */
    render: function(options)
    {
        if (!this.rendered) {
            this.rendered = true;
            this._super('render', [options]);
        }
    },

    /**************************************** RENDERS DIALOG LIST *******************************************/
    /**
     * Renders Dialog item. Appends it to the Dashlet layout. Fires handlers after rendering.
     * @param model
     */
    renderDialog: function(model){
        this.processNeedToDoIcons(model);

        var template = app.template.getView(this.name + '.dialog-item');
        if (template) {


            this.$el.find('.ntdv-collab-dialog-list').append(template({
                model: model
            }));

            var $dialogContainer = this.$el.find('.ntdv-collab-dialog-container[data-id="'+model.get('id')+'"]');
            this.calculateDialogDataAndActionsWidth($dialogContainer);
        }

        this.addBrokenNeedToDoEventsImageHandler();
    },

    /**
     * Calculates dialog width for proper dialog data styling
     * @param $dialogContainer
     */
    calculateDialogDataAndActionsWidth: function($dialogContainer){
        if(!$dialogContainer || !$dialogContainer.length)
            return;
        var dialogActionsWidth = $dialogContainer.find('.ntdv-collab-dialog-actions').width(),
            $dialogDataContainer = $dialogContainer.find('.ntdv-collab-message-data');
        if(dialogActionsWidth > 0){
            $dialogDataContainer.css('width', 'calc(100% - '+dialogActionsWidth+'px)')
        }
    },

    /**
     * Forms Icon Url for Need To Do Event if necessary
     * @param model
     */
    processNeedToDoIcons: function (model) {
        if (!model.get('event_icon_url')) {
            var iconUrl = this.buildNeedToDoEventIconUrl(model.get('needtodo_event_type_id'));
            model.set('event_icon_url', iconUrl);
        }
    },

    buildNeedToDoEventIconUrl: function(eventId) {
        return app.api.buildFileURL({
            module: 'PX_NeedToDoEventTypes',
            id: eventId,
            field: 'icon'
        });
    },

    /**
     * Handlers broken icons for Need To Do Events
     * @private
     */
    addBrokenNeedToDoEventsImageHandler: function() {
        this.$('.ntdv-collab-event-icon img').on('error', _.bind(function(event) {
            var $brokenImg = $(event.currentTarget);
            $brokenImg.attr('src', 'custom/PxRoutines/NeedToDoViewer/PX_NeedToDoEvents/img/broken_image_icon.png');
        }, this));
    },

    /**************************************** RENDERS|TOGGLE DIALOG HISTORY *******************************************/
    /**
     * Load and toggle Dialog on ToggleButton click
     * @param e
     */
    loadDialogHistory: function(e){
        var $el = $(e.currentTarget),
            id = $el.data('id');

        if(!_.isUndefined(this.dialogHistory[id])){
            this.toggleDialog(e);
        }else{
            this.dialogHistory[id] = this.dialogHistory[id] || [];
            this.loadDialog(id, e);
        }
    },

    /**
     *  Load and display messages for particular dialog
     * @param id
     * @param e
     */
    loadDialog: function(id, e){
        var url = app.api.buildURL('px-need-to-do-collaboration/dialog-list/' + id, '', {}, {});

        app.api.call('read', url, {}, {
                success: _.bind(function (response) {
                    var $el = $(e.currentTarget),
                        $container = $el.closest('.ntdv-collab-dialog-container'),
                        $dialogHistory = $container.find('.ntdv-collab-toggle-block');

                    _.each(response.records, function(item, key){
                        // response.records[key]['author_avatar_url'] = this.processMessageAuthorAvatar(item['created_by'], item['created_name'], item['created_by']);
                        response.records[key]['author_avatar_url'] = this.getAvatarUrlForUser(item['created_by'], item['created_name'], item['created_by']);
                        response.records[key]['message_text'] = this.formatMessageTagsAndLinks(item['data'] || {value: ''});
                    }, this);

                    var template = app.template.getView(this.name + '.dialog-history');
                    if (template) {
                        $dialogHistory.append(template({
                            self: this,
                            messages: response.records
                        }));
                    }

                    this.dialogHistory[id] = response.records;
                    this.toggleDialog(e);
                    this.recalculateNewMessages($container);
                    return;
                }, this),
                error: _.bind(function () {
                    this.toggleDialog(e);
                    return;
                }, this)
            });

    },

    toggleDialog: function(e){
        var $el = $(e.currentTarget),
            $dialogContainer = $el.closest('.ntdv-collab-dialog-container'),
            isExpanded = $dialogContainer.hasClass('expanded');

        if(!isExpanded) {
            this.collapseAllDialogs();
        }

        if(isExpanded){
            this.collapseDialog($dialogContainer);
        }else{
            this.expandDialog($dialogContainer);
        }
    },

    collapseDialog: function($dialogContainer){
        var $toggleTopButton = $dialogContainer.find('.ntdv-collab-toggle-dialog'),
            $toggleBlock = $dialogContainer.find('.ntdv-collab-toggle-block');

        $toggleBlock.slideToggle(500);

        $toggleTopButton.find('a').css('top', '-20px');
        $dialogContainer.removeClass('expanded');
        $toggleTopButton.removeClass('up').addClass('down');
        $dialogContainer.find('.ntdv-collab-read-msg').hide();
    },

    expandDialog: function($dialogContainer){
        var $toggleTopButton = $dialogContainer.find('.ntdv-collab-toggle-dialog'),
            $toggleBlock = $dialogContainer.find('.ntdv-collab-toggle-block');

        $toggleBlock.slideToggle(500, _.bind(function(){
            this.scrollToFirstUnread($dialogContainer);
            this.readMessages();
        }, this));

        $dialogContainer.addClass('expanded');
        $toggleTopButton.removeClass('down').addClass('up');
    },

    scrollToFirstUnread: function($dialogContainer){
        var $firstNewMessage = $dialogContainer.find('.ntdv-collab-reply-container.new:first');

        if(!$firstNewMessage.length)
            return;

        var dashletContainer = this.$el[0],
            firstNewMessage = $firstNewMessage[0],
            messageTopPosition = firstNewMessage.offsetTop; // + firstNewMessage.offsetHeight;

        if(!this.isMessageInViewport(firstNewMessage, dashletContainer)){
            var prevOffset = 0,
                $previousDialogContainer = $dialogContainer.prev('.ntdv-collab-dialog-container');
            while($previousDialogContainer.length){
                prevOffset += $previousDialogContainer[0].offsetHeight;
                $previousDialogContainer = $previousDialogContainer.prev('.ntdv-collab-dialog-container')
            }
            messageTopPosition += prevOffset;

            this.$el.animate({scrollTop:messageTopPosition}, 'slow');
        }
    },

    collapseAllDialogs: function(){
        var $openedDialogBlocks = this.$el.find('.ntdv-collab-dialog-container.expanded'),
            self = this;
        $openedDialogBlocks.each(function(){
            self.collapseDialog($(this));
        });
    },

    /**
     * Build author avatar for message
     * @param userId
     * @returns {*}
     */
    processMessageAuthorAvatar: function (userId) {
        if (!_.isUndefined(userId)) {
            return this.buildAuthorIconUrl(userId);
        }
        return '';
    },

    buildAuthorIconUrl: function(id) {
        return app.api.buildFileURL({
            module: 'Users',
            id: id,
            field: 'picture'
        });
    },

    getAvatarUrlForUser: function (userId, userFullName, commentId){
        var hasPicture = this.checkUserHasPicture(userId, userFullName, commentId);

        return hasPicture ? this.buildAvatarUrl(userId) : '';
    },

    checkUserHasPicture: function (userId, userFullName, commentId) {
        var hasPicture;

        // If processing the current user's avatar, no need to fetch
        if (userId === app.user.get('id')) {
            hasPicture = !_.isEmpty(app.user.get('picture'));
        } else {
            // Check cache
            hasPicture = this.getUserPictureStatus(userId);
        }

        // If not current user or cached, call api to check if user has a picture
        if (_.isUndefined(hasPicture)) {
            this.fetchUserPicture(userId, userFullName, commentId);
            hasPicture = false; // Use placeholder until api call finishes
        }

        return hasPicture;
    },

    fetchUserPicture: function(userId, userFullName, commentId) {
        var self = this,
            user = app.data.createBean('Users', {id: userId});

        user.fetch({
            fields: ["picture"],
            success: function () {
                var pictureUrl = self.buildAvatarUrl(userId),
                    hasPicture = !_.isEmpty(user.get('picture'));

                self.setUserPictureStatus(userId, hasPicture);

                // If picture exists, replace the activity image with the user's profile picture
                if (hasPicture) {
                    var $commentContainer = self.$el.find('.ntdv-collab-reply-container [data-id="'+commentId+'"]');
                    $commentContainer.find('.ntdv-collab-reply-author a').html("<img src='" + pictureUrl + "' alt='" + userFullName + "'>");
                }
            },
            error: function () {
                // Problem retrieving picture, use placeholder
                self.setUserPictureStatus(userId, false);
            }
        });
    },

    setUserPictureStatus: function(userId, hasPicture) {
        app.cache.set("user:avatars:" + userId, hasPicture);
        app.cache.set("user:avatars:" + userId + ":expiry", $.now() + 36000000);
    },

    getUserPictureStatus: function(userId) {
        var hasPicture = app.cache.get("user:avatars:" + userId),
            cachedTTL = app.cache.get("user:avatars:" + userId + ":expiry");

        return (cachedTTL < $.now()) ? undefined : hasPicture;
    },

    buildAvatarUrl: function(userId) {
        return app.api.buildFileURL({
            module: 'Users',
            id: userId,
            field: 'picture'
        });
    },





















    /**
     * Format message text for display
     * @param data
     * @returns {*}
     */
    formatMessageTagsAndLinks: function(data) {
        if (data) {
            data.value = this.formatTags(data.value);
            data.value = this.formatLinks(data.value);
        }

        return data;
    },

    formatTags: function(text) {
        var html = '',
            tagRegExp = /@\[([\w]+):([\d\w\-]+):(.+?)\]/g,
            tagTemplate = Handlebars.compile('<span class="label label-{{module}} sugar_tag"><a href="#{{buildRoute module=module id=id}}">{{name}}</a></span>');

        if (text && (text.length > 0)) {
            html = text.replace(tagRegExp, function(str, module, id, name) {
                name = new Handlebars.SafeString(name);
                module = (module === 'Users') ? 'Employees' : module;
                return tagTemplate({module: module, id: id, name: name});
            });
        }

        return $.trim(html);
    },

    formatLinks: function(text) {
        var formattedText = '';

        if (text && (text.length > 0)) {
            var urlRegExp = /\b((?:https?:\/\/|www\d{0,3}[.]|[a-z0-9.\-]+[.][a-z]{2,4}\/)(?:[^\s()<>]+|\(([^\s()<>]+|(\([^\s()<>]+\)))*\))+(?:\(([^\s()<>]+|(\([^\s()<>]+\)))*\)|[^\s`!()\[\]{};:\'".,<>?«»“”‘’]))/ig;
            formattedText = text.replace(urlRegExp, function(url) {
                var href = url;
                if ((url.indexOf('http://') !== 0) && (url.indexOf('https://') !== 0)) {
                    href = 'http://' + url;
                }
                return '<a href="' + href + '" target="_blank">' + url + '</a>';
            });
        }

        return formattedText;
    },

    /**************************************** ON SCROLL EVENT & READ MESSAGE & STICK TOP TOGGLE BUTTON *******************************************/

    onScroll: function(){
        this.stickDialogToggleButton();
        this.readMessages();
    },

    /**
     * Read messages from viewport
     */
    readMessages: function(){

        var $dialogContainer = this.$el.find('.ntdv-collab-dialog-container.expanded');
        if(!$dialogContainer.length) {
            return;
        }

        var self = this,
            $messageContainers = $dialogContainer.find('.ntdv-collab-reply-container.new');
        $messageContainers.each(function(){
            var $messageContainer = $(this),
                inViewPort = self.isMessageInViewport($messageContainer[0], self.$el[0]);

            if(inViewPort){
                self.readMessage($messageContainer);
            }
        });
    },

    /**
     * Defines whether message is visible or not.
     * Message is considered visible if it has >=50% in viewport.
     * @param message
     * @param container
     * @returns {boolean}
     */
    isMessageInViewport: function(message, container){
        var top = message.offsetTop,
            height = message.offsetHeight;

        var containerTop = container.offsetTop + container.scrollTop,
            containerHeight = container.offsetHeight;
        while(message.offsetParent) {
            message = message.offsetParent;
            top += (message.offsetTop);
        }
        while(container.offsetParent) {
            container = container.offsetParent;
            containerTop += (container.offsetTop);
        }

        var isTopVisible = (top - (height/2)) >= (containerTop) && (top) <= (containerTop + containerHeight),
            isBottomVisible = (top + (height/2)) <= (containerTop + containerHeight);

        return isTopVisible && isBottomVisible;
    },

    /**
     * Mark message as READ. Recalculate unread messages number.
     * Fade out "New message" flag, after message was read.
     * @param $messageContainer
     */
    readMessage: function($messageContainer){
        var messageId = $messageContainer.data('id');

        if (!messageId.length || $messageContainer.hasClass('in-process') || !$messageContainer.hasClass('new'))
            return;

        $messageContainer.addClass('in-process');
        var url = app.api.buildURL('px-need-to-do-collaboration/read-message/' + messageId, '', {}, {});
        app.api.call('update', url, {},
            {
                success: _.bind(function (response) {
                    if(response.success){
                        $messageContainer.removeClass('in-process').removeClass('new');
                        var $dialogContainer = $messageContainer.closest('.ntdv-collab-dialog-container');
                        this.recalculateNewMessages($dialogContainer);

                        var $newMessageDot = $messageContainer.find('.ntdv-collab-new-msg'),
                            $readMessageDot = $messageContainer.find('.ntdv-collab-read-msg');
                        setTimeout(function() {
                            $newMessageDot.fadeOut(300);
                            $readMessageDot.fadeIn(300);
                        }, 100);
                    }
                }, this),
                error: _.bind(function () {
                    return;
                }, this)
            });
    },

    /**
     * Recalculate New Messages number detecting New Message Flag in dialog history.
     * Hide New Message Count block after reading all messages.
     * @param $dialogContainer
     */
    recalculateNewMessages: function($dialogContainer){
        var $newMessagesContainer = $dialogContainer.find('.ntdv-collab-dialog-new-msg-block'),
            $newMessagesNumber = $dialogContainer.find('.ntdv-collab-dialog-new-msg-count'),
            newMessageContainer = $dialogContainer.find('.ntdv-collab-reply-container.new'),
            count = newMessageContainer.length;

        if(!count)
            $newMessagesContainer.fadeOut(300);
        else
            $newMessagesNumber.html(count);
    },

    stickDialogToggleButton: function(){
        var $el = this.$el,
            $dialogContainer = $el.find('.ntdv-collab-dialog-container.expanded');
        if(!$dialogContainer.length){
            return;
        }
        var $topEl = $dialogContainer.find('.ntdv-collab-toggle-dialog'),
            $bottomEl = $dialogContainer.find('.ntdv-collab-collapse-dialog'),
            topElViewPort = this.isElementInViewport($topEl[0], $el[0]),
            bottomElViewPort = this.isElementInViewport($bottomEl[0], $el[0]);

        if(!topElViewPort.isTopVisible && !topElViewPort.isBelow && !bottomElViewPort.isAbove){
            var topPosition;

            var prevOffset = 0,
                $previousDialogContainer = $dialogContainer.prev('.ntdv-collab-dialog-container');
            while($previousDialogContainer.length){
                prevOffset += $previousDialogContainer[0].offsetHeight;
                $previousDialogContainer = $previousDialogContainer.prev('.ntdv-collab-dialog-container')
            }

            topPosition = $el[0].scrollTop - $topEl[0].offsetTop - prevOffset;

            $topEl.find('a').css('top', topPosition);
        }else{
            $topEl.find('a').css('top', '-20px');
        }
    },

    isElementInViewport: function(el, parent){
        var top = el.offsetTop - 20,
            height = el.offsetHeight;

        var parentTop = parent.offsetTop + parent.scrollTop,
            parentHeight = parent.offsetHeight;
        while(el.offsetParent) {
            el = el.offsetParent;
            top += (el.offsetTop);
        }
        while(parent.offsetParent) {
            parent = parent.offsetParent;
            parentTop += (parent.offsetTop);
        }
        var isInViewPort = {
            isTopVisible : (top) >= (parentTop) && (top) <= (parentTop + parentHeight),
            isBottomVisible: (top + height) <= (parentTop + parentHeight),
            isAbove: (top + height) <= parentTop,
            isBelow: top >= (parentTop + parentHeight),
            top: top,
            bottom: (top + height)
        };
        return isInViewPort;
    },

    /**************************************** MEW MESSAGE (REPLY) *******************************************/

    changeTaggableRecord: function(e) {
        var $el = $(e.currentTarget),
            $dialog = $el.closest('.ntdv-collab-dialog-container'),
            contextModule = 'PX_NeedToDoEvents',
            contextId = $dialog.data('needtodo-id');

        this.setTaggableRecord(contextModule, contextId);

    },
    /**
     * Fires on ReplyField change.
     * @param e
     */
    checkPlaceholder: function(e) {
        this.changeTaggableRecord(e);

        var el = e.currentTarget;
        if (el.textContent) {
            el.setAttribute('data-hide-placeholder', 'true');
            this.toggleReplyButton(true);

            // if button is Enter, we post the message
            if ((e.which == 13 || e.which == 10) && e.ctrlKey){
                var $dialogContainer = $(el).closest('.ntdv-collab-dialog-container');
                this.addReply($dialogContainer);
                e.preventDefault();
                return false;
            }
        } else {
            el.removeAttribute('data-hide-placeholder');
            this.toggleReplyButton(false);
        }
    },

    /**
     * Inserts html into content editable div. Used for adding new line on Ctrl+Enter click.
     * @param html
     */
    // insertHtmlIntoContentEditableDiv: function(html){
    //     console.info('->> insert!');
    //     var sel, range;
    //     if (window.getSelection) {
    //         // IE9 and non-IE
    //         sel = window.getSelection();
    //         if (sel.getRangeAt && sel.rangeCount) {
    //             range = sel.getRangeAt(0);
    //             range.deleteContents();
    //
    //             // Range.createContextualFragment() would be useful here but is
    //             // non-standard and not supported in all browsers (IE9, for one)
    //             var el = document.createElement("div");
    //             el.innerHTML = html;
    //             var frag = document.createDocumentFragment(), node, lastNode;
    //             while ( (node = el.firstChild) ) {
    //                 lastNode = frag.appendChild(node);
    //             }
    //             range.insertNode(frag);
    //
    //             // Preserve the selection
    //             if (lastNode) {
    //                 range = range.cloneRange();
    //                 range.setStartAfter(lastNode);
    //                 range.collapse(true);
    //                 sel.removeAllRanges();
    //                 sel.addRange(range);
    //             }
    //         }
    //     } else if (document.selection && document.selection.type != "Control") {
    //         // IE < 9
    //         document.selection.createRange().pasteHTML(html);
    //     }
    // },

    /**
     * Activates/disactivates the ReplyButton
     * @param enable
     */
    toggleReplyButton: function(enable) {
        if (!enable) {
            this.disableReplyButton();
        } else {
            this.enableReplyButton();
        }
    },

    /**
     * Enable Submit button
     */
    enableReplyButton: function() {
        this.$('.reply-btn')
            .removeClass('disabled')
            .attr('aria-disabled', false)
            .attr('tabindex', 0);
    },

    /**
     * Disable Submit button
     */
    disableReplyButton: function() {
        this.$('.reply-btn')
            .addClass('disabled')
            .attr('aria-disabled', true)
            .attr('tabindex', -1);
    },

    replyButtonClick: function(e){
        var el = e.currentTarget,
            $dialogContainer = $(el).closest('.ntdv-collab-dialog-container');
        this.addReply($dialogContainer);
    },

    /**
     * Adds new Reply to the Dialog.
     * @param e
     */
    addReply: function ($dialogContainer) {
        var self = this,
            parentType = 'PX_NeedToDoViewerComments',
            parentId = $dialogContainer.data('id'),
            payload = {
                parent_type: parentType,
                parent_id: parentId,
                data: {},
                name: ''
            },
            bean;

        payload.data = this.getReplyData($dialogContainer);

        if (!this.messageIsProcessing[parentId] && payload.data.value && (payload.data.value.length > 0)) {
            this.messageIsProcessing[parentId] = true;
            if(_.isUndefined(this.topicModel[parentId])){
                this.topicModel[parentId] = app.data.createBean('PX_NeedToDoViewerComments', {id: parentId});

                this.topicModel[parentId].fetch({
                    success: function () {
                        bean = app.data.createRelatedBean(self.topicModel[parentId], null, 'comments');
                        bean.save(payload, {
                            relate: true,
                            success: _.bind(self.addReplyCallback, self, $dialogContainer)
                        });
                    },
                    error: function () {
                        self.topicModel[parentId] = null;
                    }
                });
            }else{
                bean = app.data.createRelatedBean(this.topicModel[parentId], null, 'comments');
                bean.save(payload, {
                    relate: true,
                    success: _.bind(self.addReplyCallback, self, $dialogContainer)
                });
            }
        }
    },

    /**
     * Returns inputted reply
     * @param $container
     * @returns {*|{value, tags}}
     */
    getReplyData: function($container) {
        return this.unformatTags($container.find('div.reply-body-value'));
    },

    /**
     * Adds reply to html after saving
     * @param $container
     * @param model
     */
    addReplyCallback: function ($container, model) {
        $container.find('div.reply-body-value').empty().trigger('change');

        var template = app.template.getView(this.name + '.message'),
            data = {
                author_avatar_url: this.getAvatarUrlForUser(app.user.get('id'), app.user.get('full_name'), model.get('id')),
                author_name: model.get('created_by_name'),
                created_by: model.get('created_by'),
                data: model.get('data'),
                message_text: this.formatMessageTagsAndLinks(model.get('data')),
                date_entered: model.get('date_entered'),
                id: model.get('id'),
                is_new: 0,
                name: "",
                was_created: true
            };
        this.dialogHistory[model.get('parent_id')] = this.dialogHistory[model.get('parent_id')] || [];
        this.dialogHistory[model.get('parent_id')].push(data);

        if (template) {
            $container.find('.ntdv-collab-create-message').before(template({
                self: this,
                data: data
            }));
        }

        this.messageIsProcessing[model.get('parent_id')] = false;
    },

    /**************************************** HANDLEBAR HELPERS *******************************************/
    initHandleBarHelpers: function(){

    },

    /**************************************** CLEAN UP *******************************************/
    // _dispose: function() {
    //     console.info('DISPOSE');
    //     // app.view.View.prototype._dispose.call(this);
    //
    //     this._super('_dispose');
    //     this.dialogHistory = {};
    // },

    unbindData: function() {
        var model, collection;

        if (this.context.parent) {
            model = this.context.parent.get('model');
            collection = this.context.parent.get('collection');

            if (model) {
                model.off('change sync', null, this);
            }
            if (collection) {
                collection.off('sync', null, this);
            }
        }

        this.disposeAllHistory();
        this._super('unbindData');
    },

    disposeAllHistory: function(){
        this.dialogHistory = {};
        this.$el.find('.ntdv-collab-dialog-list').html('');
    },

    /**************************************** SHOW STATISTICS *******************************************/

    showMessageStatistics: function(e){
        var $el = $(e.currentTarget),
            $messageContainer = $el.closest('.ntdv-collab-reply-container'),
            id = $messageContainer.data('id');

        var url = app.api.buildURL('px-need-to-do-collaboration/statistics/' + id, '', {}, {});

        app.api.call('read', url, {}, {
            success: _.bind(function (response) {
                _.each(response.participants, _.bind(function(item, key){
                    response.participants[key]['participant_avatar_url'] = this.processMessageAuthorAvatar(item['participant_id']);
                }, this))

                var view = app.view.createView({
                    context: this.context,
                    type: "px-ntdv-collaboration-dashlet",
                    layout: this,
                    data: response
                });
                view.$el.prependTo(app.alert.$alerts);
                view.render();

                return;
            }, this),
            error: _.bind(function () {
                return;
            }, this)
        });
    },

    /**************************************** Favorite (Unfavorite) Dialog *******************************************/

    toggleDialogAsFavorite: function(e){
        var $el = $(e.currentTarget),
            $dialogContainer = $el.closest('.ntdv-collab-dialog-container'),
            dialogId = $dialogContainer.data('id');

        var model = _.find(this.collection.models, function(dialogModel){
            return dialogModel.get('id') == dialogId;
        }, this);

        if(_.isEmpty(model))
            return;

        var isFollowing = model.get("following");
        if(!_.isUndefined(isFollowing)) {
            var options = {
                alerts: false
            };
            if (model.follow(!isFollowing, options) === false) {
                app.logger.error('Unable to follow "' + model.module + '" record "' + model.id);
                return;
            }else{
                var classToRemove = isFollowing ? 'fa-star' : 'fa-star-o',
                    classToAdd = isFollowing ? 'fa-star-o' : 'fa-star';
                $dialogContainer.find('.ntdv-collab-dialog-favorite i').removeClass(classToRemove).addClass(classToAdd);
            }
        }
    },

    /**************************************** ADD LICENSING *******************************************/
    _render: function() {
        this._super('_render');
        // this.checkCurrentUserLicenseAccess();
    },

    checkCurrentUserLicenseAccess: function(){
        if(this.licenseStatus === null){
            this.licenseStatus = undefined;
            var url = app.api.buildURL('PX_NeedToDoEvents', 'license/validate');
            app.api.call('read', url, null, {
                success: _.bind(function(){
                    this.licenseStatus = true;
                    this.onLicenseValidation(true, '');
                }, this),
                error: _.bind(function (response) {
                    this.licenseStatus = false;
                    this.licenseErrorLabel = response.message;
                    this.onLicenseValidation(false, response.message);
                }, this)
            });
        }else if(this.licenseStatus === false || this.licenseStatus === true){
            this.onLicenseValidation(this.licenseStatus, this.licenseErrorLabel);
        }
    },

    onLicenseValidation: function(status, label){
        if(status){
            this.$('.block-footer').html(app.lang.get('LBL_NO_DATA_AVAILABLE'));
        }else{
            this.$('.block-footer').html(this.licenseErrorLabel);
        }
    },

})

