<?php

class PX_NeedToDoEventTypesModuleApi extends ModuleApi {

    public function registerApiRest() {
        return array(
            'enum' => array(
                'reqType' => 'GET',
                'path' => array('PX_NeedToDoEventTypes','enum', 'target_attribute_name', '?'),
                'pathVars' => array('module', 'enum', 'field', 'target_module'),
                'method' => 'getAttributesForTargetModule',
                'shortHelp' => 'This method returns enum values for a specified field',
                'longHelp' => 'include/api/help/module_enum_get_help.html',
            ),
            'lb-template-arguments-enum' => array(
                'reqType' => 'GET',
                'path' => array('PX_NeedToDoEventTypes','enum', 'lb_template_arguments_list', '?'),
                'pathVars' => array('module', 'enum', 'field', 'lb_template_id'),
                'method' => 'getLBTemplateArgumentsEnum',
                'shortHelp' => 'This method returns enum values for a specified field',
                'longHelp' => 'include/api/help/module_enum_get_help.html',
            ),
            'lb-template-arguments-values' => array(
                'reqType' => 'GET',
                'path' => array('PX_NeedToDoEventTypes','enum', 'lb_template_arguments_values', '?'),
                'pathVars' => array('module', 'enum', 'field', 'target_module'),
                'method' => 'getLBTemplateArgumentsValues',
                'shortHelp' => 'This method returns enum values for a specified field',
                'longHelp' => 'include/api/help/module_enum_get_help.html',
            ),
            'target_entity_parent_list' => array(
                'reqType' => 'GET',
                'path' => array('PX_NeedToDoEventTypes','enum', 'target_entity_parent_list', '?'),
                'pathVars' => array('module', 'enum', 'field', 'target_module'),
                'method' => 'getTargetEntityParentList',
                'shortHelp' => 'This method returns enum values for a specified field',
                'longHelp' => 'include/api/help/module_enum_get_help.html',
            ),
        );
    }

    public function getAttributesForTargetModule(ServiceBase $api, array $args)
    {
        $this->requireArgs($args, array('field'));

        $bean = BeanFactory::newBean($args['module']);

        if(!isset($bean->field_defs[$args['field']])) {
            throw new SugarApiExceptionNotFound('field not found');
        }

        $vardef = $bean->field_defs[$args['field']];

        $value = null;
        $cache_age = 0;

        if(isset($vardef['function'])) {
            $cache_age = 60;
        } else {
            $cache_age = 3600;
        }
        if(empty($args['target_module']))
            $value = [];
        else {
            $targetModule = $args['target_module'];
            $value = PX_NeedToDoEventTypes::getTargetAttributeList($targetModule);
//            $value = getOptionsFromVardef($vardef);
        }
        if ($value === false) {
            throw new SugarApiExceptionNotFound('options not found');
        }
        // If a particular field has an option list that is expensive to calculate and/or rarely changes,
        // set the cache_setting property on the vardef to the age in seconds you want browsers to wait before refreshing
        if(isset($vardef['cache_setting'])) {
            $cache_age = $vardef['cache_setting'];
        }
        generateEtagHeader(md5(serialize($value)), $cache_age);
        return $value;
    }

    public function getLBTemplateArgumentsEnum(ServiceBase $api, array $args)
    {
        $this->requireArgs($args, array('lb_template_id'));

        $bean = BeanFactory::newBean($args['module']);

//        if(!isset($bean->field_defs[$args['field']])) {
//            throw new SugarApiExceptionNotFound('field not found');
//        }

//        $vardef = $bean->field_defs[$args['field']];

        $value = null;
        $cache_age = 0;

//        if(isset($vardef['function'])) {
            $cache_age = 60;
//        } else {
//            $cache_age = 3600;
//        }
        if(empty($args['lb_template_id']))
            $value = [];
        else {
            $lbTemplateID = $args['lb_template_id'];
            $value = PX_NeedToDoEventTypes::getLBTemplateArguments($lbTemplateID);
//            $value = getFunctionValue(null, $vardef['function'], array(
//                'lb_template_id' => $targetModule
//            ));
//            $value = getOptionsFromVardef($vardef);
        }
        if ($value === false) {
            throw new SugarApiExceptionNotFound('options not found');
        }
        // If a particular field has an option list that is expensive to calculate and/or rarely changes,
        // set the cache_setting property on the vardef to the age in seconds you want browsers to wait before refreshing
//        if(isset($vardef['cache_setting'])) {
//            $cache_age = $vardef['cache_setting'];
//        }
        generateEtagHeader(md5(serialize($value)), $cache_age);
        return $value;
    }

    public function getLBTemplateArgumentsValues(ServiceBase $api, array $args)
    {
        $this->requireArgs($args, array('target_module'));

        $value = null;
        $cache_age = 60;

        if(empty($args['target_module']))
            $value = [];
        else {
            $targetModule = $args['target_module'];
            $value = PX_NeedToDoEventTypes::getTargetAttributeValues($targetModule);
        }
        if ($value === false) {
            throw new SugarApiExceptionNotFound('options not found');
        }
        generateEtagHeader(md5(serialize($value)), $cache_age);
        return $value;
    }

    public function getTargetEntityParentList(ServiceBase $api, array $args)
    {
        $this->requireArgs($args, array('target_module'));

        $value = null;
        $cache_age = 60;

        if(empty($args['target_module']))
            $value = [];
        else {
            $targetModule = $args['target_module'];
//            $value = PX_NeedToDoEventTypes::getTargetAttributeValues($targetModule);
            $targetEntityParentList = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Helpers\TargetEntityParentList();
            $value = $targetEntityParentList->getRelateFields($targetModule);
        }
        if ($value === false) {
            throw new SugarApiExceptionNotFound('options not found');
        }
        generateEtagHeader(md5(serialize($value)), $cache_age);
        return $value;
    }
}
