({
    extendsFrom: "BaseField",
    rowId: 0,

    formRowTemplate: null,

    hasSystemName : false,

    notifiedOfParseError: false,

    fields: [],

    initialize: function(arguments){
        this._super("initialize", [arguments]);

        this.events = _.extend(this.events, {
            'click [px-context-action=add]': 'addRow',
            'click [px-context-action=remove]': 'removeRow',
        });

        this.formRowTemplate = app.template.getField(this.type, 'notify-user', this.module);

        this.fields = [];
        this.notifiedOfParseError = false;
    },

    bindDataChange: function() {
        if (this.model) {
            this.model.on(/*"change:lb_template_name " + */"change:" + this.name, function(model, value) {
                this.render();
            }, this);
        }
    },

    _render: function(values) {
        this.rowId = 0;
        this._loadTemplate();
        this.$el.html(this.template(this) || '');

        this.renderFields(values);
        // this.displayInsertButton();

    },


    displayInsertButton: function(){
        if(this.disposed || _.isEmpty(this.$el) || !this.$el.find('.px-context-actions').length)
            return;

        if(this.action == 'detail')
            this.$el.find('.px-context-actions').hide();
        else
            this.$el.find('.px-context-actions').show();
    },

    renderFields: function(values){
        var defs = values || this.model.get(this.name);

        if(!_.isEmpty(defs) && _.isString(defs)){
            try{
                defs = JSON.parse(defs);
            }catch(err){
                defs = null;
                var errorMsg = app.lang.get('LBL_CONTEXT_PARSE_ERROR', 'PX_FlowchartTemplates');
                app.logger.warn(errorMsg);
                if(!this.notifiedOfParseError) {
                    this.notifiedOfParseError = true;
                    app.alert.show('context_parse_error', {
                        level: 'warning',
                        messages: errorMsg
                    });
                }
            }
        }

        if(_.isEmpty(defs) || (_.isArray(defs) && defs.length <= 0) || (_.isString(defs) && defs == '[]')){
            this.renderRow(null, {});
            return;
        }

        var self = this;
        _.each(defs, function(data, key){
            self.renderRow(null, data);
        });
    },

    getRowSequenceNumber: function(data){
        if(_.isUndefined(data['key'])){
            return ++this.rowId;
        }

        if(data['key'] > this.rowId)
            this.rowId = data['key'];

        return data['key'];
    },

    addRow: function(e){
        var $row;
        if (e) {
            $row = this.$(e.currentTarget).closest('[data-container=notify-user]');
            $row.after(this.formRowTemplate({hasSystemName: this.hasSystemName}));
            $row = $row.next();
        }

        this.renderRow($row, {});

        var rowInput = $row.find('input[type="text"]');
        if(rowInput.length)
            rowInput.focus();
    },

    removeRow: function(e){
        var $row = this.$(e.currentTarget).closest('[data-container=notify-user]');

        this.removeModelValues($row);

        $row.remove();
        if (this.$('[data-container=notify-user]').length === 0) {
            this.addRow();
        }
    },

    removeModelValues: function($row){
        var data = $row.data(),
            linkLevelOneName = data['linkLevel1'];
            // ,keyName = data['value'];

        this.model.set(linkLevelOneName, '');
        // this.model.set(keyName, '');
    },

    renderRow: function($row, data){
        $row = $row || $(this.formRowTemplate()).appendTo(this.$el);
        data = data || {};

        var rowId = this.getRowSequenceNumber(data);

        this.renderRelatedField($row, data, rowId);
        this.renderNotifiedFlagField($row, data, rowId);
        // this.renderLinkField($row, data, rowId);
        // this.renderValueAttribute($row, data, rowId);

        $row.data('key', rowId);
    },

    renderRelatedField: function($row, data, rowId){
        var field, fieldContainer, fieldEl,
            fieldName = this.getRelatedFieldName(rowId),
            relateIdName = this.getRelatedIdFieldName(rowId),
            fieldValue = data.name || '',
            fieldIdValue = data.id || '';
            // fieldPlaceholderLabelKey = 'LBL_PARENT_LINK_LABEL',
            // selectPlaceholderLabel = "Set %FIELD_PLACEHOLDER%...".replace('%FIELD_PLACEHOLDER%', app.lang.get(fieldPlaceholderLabelKey, this.module));

        $row.data('key', rowId);
        this.model.set(fieldName, fieldValue);
        this.model.set(relateIdName, fieldIdValue);
        var fieldDefs = _.extend({
            name: fieldName,
            type: 'relate',
            id_name: relateIdName,
            module: 'Users',
            rname: 'full_name',
            disabled: true,
            layout: this,
            readonly: true, //this.def.readonly,

            placeholder: 'LBL_TPL_TEMPLATE_KEY',
            tooltip: 'LBL_TPL_TEMPLATE_KEY',
            display: true
        }, {});
        // return;
        field = this.createField(this.model, fieldDefs);
        fieldContainer = $row.find('[data-container=relate-field]');
        fieldEl = $(field.getPlaceholder().string);
        fieldEl.appendTo(fieldContainer);
        this.renderField(field, fieldEl);
        this.fields.push(field);
    },

    renderNotifiedFlagField: function($row, data, rowId){
        var notifiedFlag = (data.notified ? true: false);
        if(!notifiedFlag){
            $row.find('[data-container=relate-field]').removeClass('span6').addClass('span12');
            $row.find('[data-container=notified-flag]').hide();
            return;
        }

        var field, fieldContainer, fieldEl,
            fieldName = this.getNotifiedFlagName(rowId),
            fieldValue = (data.notified == true ? '(Notified)' : ''),
            fieldLabel = 'LBL_PARENT_LINK_NAME';

        $row.data('key', fieldValue);
        this.model.set(fieldName, fieldValue);
        var fieldDefs = _.extend({
            name: fieldName,
            type: 'varchar',
            disabled: true,
            layout: this,
            readonly: true, //this.def.readonly,

            placeholder: 'LBL_TPL_TEMPLATE_KEY',
            tooltip: 'LBL_TPL_TEMPLATE_KEY',
            display: true
        }, {});
        // return;
        field = this.createField(this.model, fieldDefs);
        fieldContainer = $row.find('[data-container=notified-flag]');
        fieldEl = $(field.getPlaceholder().string);
        fieldEl.appendTo(fieldContainer);
        this.renderField(field, fieldEl);
        this.fields.push(field);
    },

    renderValueAttribute: function($row, data, rowId){
        var fieldDefs, field,
            fieldContainer, fieldEl,
            fieldKeyName = this.getAttributeSystemName(rowId),
            fieldKeyValue = data.name || '';

        $row.data('value', fieldKeyValue);
        this.model.set(fieldKeyName, fieldKeyValue);

        fieldDefs = _.extend({
            name: fieldKeyName,
            type: 'enum',
            parentFieldName: this.name,
            argumentPart: 'target-field',
            disabled: true,
            layout: this
        }, this.def.options.name || {});

        field = this.createField(this.model, fieldDefs);
        fieldContainer = $row.find('[data-container=px-context-key]');
        fieldEl = $(field.getPlaceholder().string);
        fieldEl.appendTo(fieldContainer);

        this.renderField(field, fieldEl);
        this.fields.push(field);
    },

    getAttributeSystemName: function(key){
        return 'attr_sys_name' + key
    },

    getRelatedFieldName: function(key){
        return this.name + '_' + key
    },

    getRelatedIdFieldName: function(key){
        return this.name + '_id_' + key
    },

    getNotifiedFlagName: function(key){
        return 'notified_' + key
    },

    createField: function(model, def) {
        var obj = {
            meta: {
                view: 'edit'
            },
            def: def,
            model: model,
            context: app.controller.context,
            viewName: 'detail', //this.action,
            view: this.view
        };

        model.fields[def.name] = def;

        var field = app.view.createField(obj);
        field.action = 'detail'; //this.action;
        return field;
    },

    renderField: function(field, $fieldEl) {
        field.setElement($fieldEl);
        try {
            field.render();
        } catch (e) {
            app.logger.error("Failed to render " + field + " on " + this.view + "\n" + e);
            app.error.handleRenderError(this.view, '_renderField', field);
        }
    },

    toggle: function(evt) {
        this.$('[data-container=notify-user]').html('');
        this.renderFields();
    },

    clearErrorDecoration: function () {
        _.each(this.fields, function(field, key) {
            if(_.isEmpty(field) || _.isEmpty(field.$el)){
                // delete this.fields[key];
            }else {
                field.clearErrorDecoration();
            }
        }, this);

        this._super('clearErrorDecoration');
    },

    beforeHandleSave: function(){
        this.model.set(this.name, this.getFormattedFieldValue());
    },

    getFieldData: function(){
        var $rows = this.$('[data-container=notify-user]');
        var contextData = [];

        _.each($rows, function(row) {
            var argumentItem = this.getArgument($(row));
            if(!_.isEmpty(argumentItem.id))
                contextData.push(argumentItem);
        }, this);

        return contextData;
    },

    getFormattedFieldValue: function(){
        var contextData = this.getFieldData();
        return JSON.stringify(contextData, null);
    },

    getArgument: function($row) {
        var data = $row.data(),
            key = data['key'],
            // fieldKeyName = this.getAttributeSystemName(key);
            relateFieldName = this.getRelatedFieldName(key),
            relateFieldIdName = this.getRelatedIdFieldName(key);
            // linkLevel1Name = this.getLinkFieldName(key);

        return {
            id: this.model.get(relateFieldIdName),
            name: this.model.get(relateFieldName),
            // linkLevel1: this.model.get(linkLevel1Name),
            // name: this.model.get(fieldKeyName),
            key: key
        };
    },

    _dispose: function() {
        _.each(this.fields, function(field){
            field.dispose();
        });
        this.fields = [];
        this._super('_dispose');
    },

    isFieldLengthValid: function(){
        if(_.isEmpty(this.model.get(this.name)))
            return true;

        var contextField = btoa(escape(this.model.get(this.name)));
        if(contextField.length >= 65535){
            return false;
        }
        return true;
    },

    getSymbolsAmountViolation: function(){
        var contextField = btoa(escape(this.model.get(this.name)));
        return contextField.length - 65535;
    },

    substringContextFieldLength: function(){
        var contextList = this.getFieldData(),
            trimmedContextList = [],
            sortedContextList = [],
            whiteListedKeyNames = [],
            newContextList = [];

        sortedContextList = _.sortBy(contextList, function(context){ return context.id.length + context.key.length; });

        _.each(sortedContextList, function(context){
            var tempArgumentsJson = '';
            trimmedContextList.push(context);

            try{
                tempArgumentsJson = btoa(escape(JSON.stringify(trimmedContextList, null)));
                if(tempArgumentsJson.length <= 65535){
                    whiteListedKeyNames.push(context.id);
                }else{
                    return;
                }
            }catch(err){
                return;
            }
        }, this);


        if(!_.isEmpty(whiteListedKeyNames)){
            _.each(contextList, function(context, key){
                if(!_.isEmpty(context) && _.indexOf(whiteListedKeyNames, context.id) !== -1){
                    newContextList.push(context);
                }
            });
        }

        try{
            this.model.set(this.name, JSON.stringify(newContextList, null));
        }catch(err){
            this.model.set(this.name, '[]');
        }
    },

    getFieldsMeta: function(){
        var contextList = this.getFieldData(),
            fieldsMeta = [];

        _.each(contextList, function(contextItem){
            fieldsMeta.push({
                'name': 'parent_relate_' + contextItem.key
            });
        }, this);

        return fieldsMeta;
    },

})
