/*
 * Your installation or use of this SugarCRM file is subject to the applicable
 * terms available at
 * http://support.sugarcrm.com/Resources/Master_Subscription_Agreements/.
 * If you do not agree to all of the applicable terms or do not have the
 * authority to bind the entity as an authorized representative, then do not
 * install or use this SugarCRM file.
 *
 * Copyright (C) SugarCRM Inc. All rights reserved.
 */
/**
 * @class View.Fields.Base.ParentField
 * @alias SUGAR.App.view.fields.BaseParentField
 * @extends View.Fields.Base.RelateField
 */
({
    extendsFrom: 'ParentField',

    flexName: 'parent_name',
    flexType: 'parent_type',
    flexId: 'parent_id',
    flexLink: 'parent',

    isFetchingOptions: false,

    /**
     * @inheritDoc
     */
    initialize: function(options) {
        this._super('initialize', [options]);

        this.flexName = this.name;
        if(!_.isEmpty(this.def['id_name']))
            this.flexId = this.def['id_name'];
        if(!_.isEmpty(this.def['type_name']))
            this.flexType = this.def['type_name'];
        if(!_.isEmpty(this.def['link_name']))
            this.flexLink = this.def['link_name'];

        this.fieldTag = 'input.select2[name=' + this.flexName + ']';
        this.typeFieldTag = 'select.select2[name=' + this.flexType + ']';

        if(!_.isEmpty(this.fieldDefs['link']))
            this.fieldDefs.link = this.flexLink;

        this._modules = null;
        this.moduleList = {};
        if(_.isObject(this.def.options)){
            this._modules = this.def.options;
            this.moduleList = {};
            this._filterModuleList(this._modules);
        }

    },

    _render: function() {
        var self = this;

        if (!this._modules || _.isEmpty(this._modules)) {
            this.loadEnumOptions(function () {
                self.isFetchingOptions = false;
                if (!self.disposed) {
                    self._super('_render', []);
                }
            });
        }else{
            this._super('_render', []);
        }

        return this;
    },

    _getRelateId: function() {
         return this.model.get(this.flexId);
     },

    format: function(value) {
        var module;
        this.def.module = this.getSearchModule();

        if (this.def.module) {
            module = app.lang.getModuleName(this.def.module);
        }

        if(!_.isUndefined(module) && module !== 'not_set') {
            var originLabel = app.lang.get(this.def.label, this.module),
                label = '';
            //if this is detail record and related entity is selected
            if(this.tplName === 'detail' && !_.isEmpty(module.trim())){
                //if label must include origin label from defs. e.g. Related to RELATED_MODULE_NAME
                if(!_.isUndefined(this.def['showOriginalLabel']) && this.def.showOriginalLabel){
                    label = originLabel + ': ' + module + '';
                }else{
                    //if label consists only of the related module name
                    label = module;
                }
            }else{
                //if this is NOT detail record OR related entity is NOT selected
                label = originLabel;
            }

            this.context.set('record_label', {
                field: this.name,
                label: label
            });
        }

        var parentCtx = this.context && this.context[this.flexType],
            setFromCtx;

        if (value) {
            this._valueSetOnce = true;
        }

        setFromCtx = !value && !this._valueSetOnce && parentCtx && _.isEmpty(this.context.get('model').link) &&
            this.view instanceof app.view.views.BaseCreateView &&
            _.contains(_.keys(app.lang.getAppListStrings(this.def[this.flexType])), parentCtx.get('module')) &&
            this.module !== this.def.module;

        if (setFromCtx) {
            this._valueSetOnce = true;
            var model = parentCtx.get('model');
            // FIXME we need a method to prevent us from doing this
            // FIXME the setValue receives a model but not a backbone model...
            var attributes = model.toJSON();
            attributes.silent = true;
            this.setValue(attributes);
            value = this.model.get(this.name);

            // FIXME we need to iterate over the populated_ that isn't working now
        }

        return app.view.fields.BaseRelateField.prototype.format.call(this, value);
    },

    /**
     * @override
     */
    setValue: function(models) {
        if (!models) {
            return;
        }
        models = _.isArray(models) ? models : [models];
        _.each(models, _.bind(function(model) {

            var silent = model.silent || false,
            // FIXME we shouldn't make this assumption and this method should
            // receive a true Backbone.Model or Data.Bean
                module = model.module || model._module;


            if (app.acl.hasAccessToModel(this.action, this.model, this.name)) {
                if (module) {
                    this.model.set(this.flexType, module, {silent: silent});
                    this._createSearchCollection();
                }
                // only set when we have an id on the model, as setting undefined
                // is causing issues with the warnUnsavedChanges() method
                if (!_.isUndefined(model.id)) {
                    this.model.set(this.flexId, model.id, {silent: silent});
                    // FIXME we shouldn't rely on model.value... and hack the full_name here until we fix it properly
                    // SC-4196 will fix this.
                    var value = model.value || model[this.def.rname || 'name'] || model['full_name'] ||
                        app.utils.formatNameLocale(model);
                    this.model.set(this.flexName, value, {silent: silent});
                }
            }
        }, this));

        // TODO we should support the auto populate of other fields like we do on normal relate.js
    },

    getSearchModule: function() {
        return this.model.get(this.flexType) || this.$(this.typeFieldTag).val();
    },

    /**
     * @inheritdoc
     * Avoid rendering process on select2 change in order to keep focus.
     */
    bindDataChange: function() {
        this._super('bindDataChange');
        if (this.model) {
            this.model.on('change:'+this.flexType, function() {
                var plugin = this.$(this.typeFieldTag).data('select2');
                if (_.isEmpty(plugin) || !plugin.searchmore) {
                    this.render();
                } else {
                    this.$(this.typeFieldTag).select2('val', this.model.get(this.flexType));
                }
            }, this);
        }
    },

    /**
     * We do not support this field for preview edit
     * @inheritdoc
     */
    _loadTemplate: function() {
        this._super('_loadTemplate');

        if (this.view.name === 'preview') {
            this.template = app.template.getField('px-parent', 'detail', this.model.module);
        }
    },

    _filterModuleList: function(modules) {
        this._super('_filterModuleList', [modules]);

        var moduleList = _.sortBy(_.pairs(this.moduleList), function(item){ return item[1]; });
        this.moduleList = {};
        // this.moduleList["not_set"] = "";
        _.each(moduleList, function(item){
            this.moduleList[item[0]] = item[1];
        }, this);
    },

    loadEnumOptions: function(callback) {
        var self = this;
        var _itemsKey = 'cache:' + this.module + ':' + this.flexType + ':items';

        this._modules = this.context.get(_itemsKey);

        if (_.isEmpty(this._modules)) {
            this.isFetchingOptions = true;
            var _key = 'request:' + this.module + ':' + this.flexType;
            if (this.context.get(_key)) {
                var request = this.context.get(_key);
                request.xhr.done(_.bind(function(o) {
                    if (this._modules !== o) {
                        this._modules = o;
                        this._filterModuleList(this._modules);
                        callback.call(this);
                    }
                }, this));
            } else {
                var request = app.api.enumOptions(self.module, self.flexType, {
                    success: function(o) {
                        if(self.disposed) { return; }
                        if (self._modules !== o) {
                            self._modules = o;
                            self._filterModuleList(self._modules);
                            self.context.set(_itemsKey, self._modules);
                            self.context.unset(_key);
                            callback.call(self);
                        }
                    }
                });
                this.context.set(_key, request);
            }
        } else if (!_.isEmpty(this.def.options)) {
            this._modules = app.lang.getAppListStrings(this.def.options);
            this._filterModuleList(this._modules);
            callback.call(this);
        }else{
            this._filterModuleList(this._modules);
            callback.call(this);
        }
    },
})
